<?php

namespace Drupal\smallads;

use Drupal\smallads\Entity\SmalladInterface;
use Drupal\user\Entity\User;
use Drupal\migrate\Event\MigratePreRowSaveEvent;
use Drupal\migrate\MigrateSkipRowException;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\block\Entity\Block;
use Drupal\migrate\Plugin\migrate\destination\EntityContentBase;

/**
 * Modify migrations before saving them.
 *
 * This is necessary for the links and more importantly the autogenerated tasks
 * Seems like a shortcoming in the routing system, but it will certainly do for
 * this module.
 */
class MigrationSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    //Migrate module might not be installed and we can't test for it here because
    //\Drupal::moduleHandler()->moduleExists('migrate_drupal') is not initialised yet!
    //so just replace the constants with their strings defined in Drupal\migrate\Event\MigrateEvents
    return [
      'migrate.pre_row_save' => ['migratePreRowSave']
    ];
  }

  /**
   * Change the field names
   */
  function migratePreRowSave(MigratePreRowSaveEvent $event) {
    $row = $event->getRow();
    $migration = $event->getMigration();
    $source = $row->getSource();

    if($migration->id() == 'd7_comment' and $row->getDestinationProperty('entity_type') == 'smallads') {
      $row->setDestinationProperty('comment_type', 'smallad');
      $row->setDestinationProperty('field_name', 'comments');
      $row->setDestinationProperty('body', $row->getDestinationProperty('comment_body'));// Assumes no comments on transactions
    }
    if($migration->id() == 'd7_comment_type') {
      if ($row->getSourceProperty('type') == 'proposition') {
        throw new MigrateSkipRowException('smallad comment types already exist');
      }
    }
    if($migration->id() == 'd7_comment_field') {
      if ($row->getSourceProperty('type') == 'proposition') {
        throw new MigrateSkipRowException('smallad comment fields already exist');
      }
    }

    // This is horrible but necessary since the entity name changed
    if ($migration->getDestinationConfiguration()['plugin']) {
      if ($row->getDestinationProperty('bundle') == 'proposition') {
        $row->setDestinationProperty('entity_type', 'smallad');
        $row->setDestinationProperty('bundle', 'offer');
      }
    }

    // Horrible but necessary if we change the entity name
    if ($migration->getDestinationConfiguration()['plugin'] == 'component_entity_display') {
      if ($row->getSourceProperty('bundle') == 'proposition') {
        throw new MigrateSkipRowException('Proposition entity display is not migratable');
      }
    }

    // Fieldable entities only.
    // If any entity used the categories field in d7, change the name of the field in this row
    if ($migration->getDestinationPlugin() instanceof EntityContentBase) {
      if ($row->hasSourceProperty('offers_wants_categories')) {
        $tids = [];
        foreach ($row->getSourceProperty('offers_wants_categories') as $delta => $val) {
          $tids[$delta]['target_id'] = $val['tid'];
        }
        $row->setDestinationProperty('categories', $tids);
        $row->setDestinationProperty('category', $tids);
        $row->removeDestinationProperty('offers_wants_categories');
      }
    }

    if ($migration->getBaseId() == 'd7_smallad') {
      $uid = $row->getSourceProperty('node_uid');
      if (!User::load($uid)) {
        throw new MigrateSkipRowException("Smallad owner user $uid no longer exists.");
      }
      // If the expiry date is before now, set the scope to private
      if ($row->getDestinationProperty('expires') < \Drupal::Time()->getRequestTime()) {
        $row->setDestinationProperty('scope', SmalladInterface::SCOPE_PRIVATE);
      }
    }

    switch ($migration->id()) {
      case 'd7_block':
        if ($row->getSourceProperty('delta') == 'add_proposition') {
          if (Block::load('addad')) {
            $block->delete();
          }
        }
      case 'd7_taxonomy_vocabulary':
        $vname = $row->getSourceProperty('machine_name');
        if ($vname == 'offers_wants_categories' or $vname == 'offers_wants_types') {
          throw new MigrateSkipRowException("$vname has an installed equavalent vocab in Smallads");
        }
        break;
      case 'd7_taxonomy_term:offers_wants_categories':
        $row->setDestinationProperty('vid', 'categories');
        break;
      case 'd7_taxonomy_term:offers_wants_types':
        throw new MigrateSkipRowException('offers_wants_types no longer exist in smallads');
      case 'd7_field':
      case 'd7_field_instance':
        $deprecated_fields = ['offers_wants_categories', 'offers_wants_types', 'comment_node_proposition'];
        if (in_array($row->getSourceProperty('field_name'), $deprecated_fields)) {
          throw new MigrateSkipRowException($row->getSourceProperty('field_name') .' is already installed by the smallads module');
        }
        if ($row->getSourceProperty('field_name') == 'comment_body'and $row->getSourceProperty('entity_type') == 'smallad') {
          throw new MigrateSkipRowException('offers_wants_types is already installed by the smallads module');
        }
        if ($row->getSourceProperty('bundle') == 'comment_node_proposition') {
          throw new MigrateSkipRowException('comment_node_proposition is implemented differently by the smallads module');
        }
        if ($row->getSourceProperty('entity_type') == 'comment' and $row->getSourceProperty('field_name') == 'comment_body') {
          throw new MigrateSkipRowException('comment_body is implemented differently by the smallads module');
        }
        break;
      case 'd7_taxonomy_vocabulary':
        if ($row->getSourceProperty('machine_name') == 'offers_wants_types') {
          throw new MigrateSkipRowException('offers_wants_types vocab is already installed by the smallads module');
        }
        break;
      case 'd7_taxonomy_term':
        //return FALSE;
        break;
      case 'd7_node_type':
        // don't migrate old node type
        if ($row->getSourceProperty('type') == 'proposition') {
          throw new MigrateSkipRowException('proposition content type is already installed by the smallads module');
        }
        break;
      case 'd7_comment_entity_display':
        if ($row->getSourceProperty('bundle') == 'proposition') {
          throw new MigrateSkipRowException('d7_comment_entity_display is already installed by the smallads module');
        }
        break;
      case 'd7_comment_field':
        if ($row->getSourceproperty('node_type') == 'proposition') {
          throw new MigrateSkipRowException('proposition comment field is already installed by the smallads module');
        }
        break;
      case 'd7_field_formatter_settings':
        if ($row->getSourceproperty('field_name') == 'offers_wants_categories') {
          $row->setDestinationProperty('field_name', 'category');
        }
        break;
      case 'd7_menu_links':
        $obselete = ['admin/config/ow', 'node/add'];
        if (in_array($row->getSourceProperty('link_path'), $obselete)) {
          throw new MigrateSkipRowException($row->getSourceProperty('link_path'). " is no longer used.");
        }
        if ($row->getSourceProperty('link_path') == 'node/add/offer') {
          $row->setSourceProperty('link_path', 'ad/add/offer');
        }
        if ($row->getSourceProperty('link_path') == 'node/add/want') {
          $row->setSourceProperty('link_path', 'ad/add/want');
        }
    }

    if ($migration->id()== 'd7_view_mode' and $row->getSourceProperty('bundle') == 'proposition') {
      throw new MigrateSkipRowException('Proposition view modes are not migratable.');
    }

  }

}
